# This file is a work of a US government employee and as such is in the Public domain. 
# Simson L. Garfinkel, March 12, 2012 

import sys
server_version = open(sys.argv[1],"r").read().strip()
print "Server Version:",server_version
if(server_version == sys.argv[2]):
    print "\n\nVersion",sys.argv[1],"is already on the server.\n\n"
    sys.exit(-1)
sys.exit(0)



try:
    from setuptools import setup, Extension, command
except ImportError:
    from distutils.core import setup, Extension

try:
    from Cython.Build import cythonize
    USE_CYTHON = True
except ImportError:
    USE_CYTHON = False

pyaff = [Extension('pyaff', ['pyaff.pyx'], libraries = ['afflib'])]

if USE_CYTHON:
    pyaff = cythonize(pyaff)

setup (name = 'PyAFF',
       version = '0.1',
       description = 'Python wrapper for AFFLIB',
       author = 'David Collett',
       author_email = 'david.collett@gmail.com',
       url = 'www.pyflag.net',
       license = "GPL",
       ext_modules = pyaff)



import sys,os;

sys.path.append(os.getenv("HOME") + "/slg/src/python")

from statbag import statbag

drives = {}                             # hash table for the drives
gb = 1000*1000*1000

class dr:
    pass

def process_afcompare(fn):
    import re
    global null_drives,total_nochg,total_null,total_lzma,total_drives,drives_larger_1g
    m = re.compile("\d\d\d\d[.]aff")
    for line in open(fn,"r"):
        r = m.search(line)
        drive = r.group(0)
        d = dr()
        fields = line.split(" ")
        while "" in fields: fields.remove("")
        d.nochg = int(fields[4])
        d.nul = int(fields[6])
        d.lzma = int(fields[8])
        d.old_bytes = int(fields[10])
        d.new_bytes = int(fields[12])
        drives[drive] = d

def process_afreport(fn):
    import re
    m = re.compile("\d\d\d\d[.]aff")
    for line in open(fn,"r"):
        r = m.search(line)
        try:
            drive = r.group(0)
            if drive not in drives: continue
            d = drives[drive]
            line = line.replace("\t"," ")
            fields = line.split(" ")
            while "" in fields: fields.remove("")
            d.imagesize  = int(fields[1])
            d.compressed = int(fields[2])
            d.uncompressed = int(fields[3])
            print d.iamgesize,d.compressed,d.uncompressed
        except AttributeError:
            pass


if(__name__=="__main__"):
    import glob
    for fn in glob.glob("*afcompare*"):
        process_afcompare(fn)
    for fn in glob.glob("*-report*"):
        process_afreport(fn)

    null_drives =0 

    big = {}

    for (fn,d) in drives.iteritems():
        if(d.nochg==0 and d.nul>=0 and d.lzma==0):
            null_drives += 1
            continue
        if(d.uncompressed<gb):
            continue

        big[fn] = d

    print "Total drives:",len(drives)
    print "Drives that were completely blank:",null_drives
    remaining = len(drives) - null_drives
    print "Drives containing some data:",remaining
    print "Drives larger than 1GB uncompressed:",len(big)
    print
    print "For the drives larger than 1GB uncompressed:"
    print "(All sizes in megabytes)"
    print "%8s %8s %8s %8s %s" % ("DriveID","Uncomp","EnCase","AFFLIB 1.7","Savings")

    def myfunc(A,B):
        a = big[A]
        b = big[B]
        if a.uncompressed < b.uncompressed: return -1
        if a.uncompressed > b.uncompressed: return 1
        return 0

    fns = big.keys()
    fns.sort(myfunc)
    avg_savings = statbag();
    for fn in fns:
        d = big[fn]
        savings = 100.0 - (100.0 * d.new_bytes/d.old_bytes)
        if(d.old_bytes/1000000 < 10): continue
        print "%8s %8d %8d %8d    %5.2f%%" % (
            fn,d.uncompressed/1000000,d.old_bytes/1000000,d.new_bytes/1000000,savings)
        avg_savings.addx(savings)
    print "\nAverage savings: %5.2f%%" % avg_savings.average()



#
# python speed test jig.
# Note: this currently doesn't work.
#

# This file is a work of a US government employee and as such is in the Public domain.
# Simson L. Garfinkel, March 12, 2012

opt_megabytes = 4096
min_range = 7
max_range = 20

#max_range = 14

import time,re,os


#def do_command(cmd):
#    """ Run a command and return the clock time """
#    print cmd
#    start = time.time()
#    os.system(cmd)
#    end = time.time()
#    return end-start

def dd_test(infile="/dev/zero",outfile="/dev/null",bs=16384):
    transfer = 1024*1024*opt_megabytes
    count = transfer / bs
    res = re.compile("(\d+) bytes/sec")
    cmd = "dd if=%s of=%s bs=%d count=%d 2>&1" % (infile,outfile,bs,count)
    print cmd
    for line in os.popen(cmd,"r"):
        print "*",line
        m = res.search(line)
        if(m):
            print "yum",m.group(1)
            return int(m.group(1))
    return False

def grep(fn,what):
    import re
    m = re.compile(what)
    for line in open(fn,"r"):
        if(m.search(line)): return line.strip()
    return False

def dd_speed_tests():
    cpu = grep("/var/run/dmesg.boot","CPU")
    mem = grep("/var/run/dmesg.boot","real memory")

    print "\n\nRunning write tests..."
    block_sizes = []
    for i in range(min_range,max_range):
        block_sizes.append(2**i)

    write_speeds = {}
    for bs in block_sizes:
        write_speeds[bs] = dd_test(outfile="speed.tmp",bs=bs) / 1000000.0

    print "\n\nRunning read tests..."
    read_speeds = {}
    for bs in block_sizes:
        read_speeds[bs] = dd_test(infile="speed.tmp",bs=bs)   / 1000000.0
    
    print "\n\n"
    print cpu
    print mem
    print "Transfer Size: ",opt_megabytes,"MB"

    def print_sorted(title,dict):
        print title
	print "bs=nn\t\tMBytes/sec"
        keys = dict.keys()
        keys.sort()
        for k in keys:
            print "%d\t\t%.2f" % (k,dict[k])

    print_sorted("Write Results\ndd if=/dev/zero of=speed.tmp\n",write_speeds)
    print_sorted("Read Results\ndd if=/speed.tmp of=/dev/null",read_speeds)


if(__name__=='__main__'):
    dd_speed_tests()



import sys
from cpython.version cimport PY_MAJOR_VERSION
cimport afflib

"""Open afflib file (or set of files)"""
def open(filename):
    return affile(filename)

"""afflib File Object"""
cdef class affile(object):
    def __cinit__(self, basestring filename):
        self.af = afflib.af_open(filename.encode(sys.getfilesystemencoding()), afflib.O_RDONLY, 0)
        if self.af == NULL:
            raise IOError("Failed to initialise afflib")

        self.size = afflib.af_get_imagesize(self.af)

    """Read data from file"""
    def read(self, int size=-1):
        readlen = size if 0 <= size <= self.size else self.size

        retdata = bytearray(readlen)
        written = afflib.af_read(self.af, retdata, size)
        if written != readlen:
            raise IOError("Failed to read all data: wanted {}, got {}".format(readlen, written))

        return bytes(retdata)

    """Seek within a file"""
    def seek(self, int offset, int whence=0):
        if afflib.af_seek(self.af, offset, whence) < 0:
            raise IOError("libaff_seek_offset failed")

    """Retrieve an aff segment by name"""
    def get_seg(self, basestring segname):
        cdef size_t buflen = 0
        cdef _segname = segname.encode('ascii')

        if afflib.af_get_seg(self.af, _segname, NULL, NULL, &buflen) != 0:
            raise IOError("error reading libaff segment")
        print(buflen)

        retdata = bytearray(buflen)

        if afflib.af_get_seg(self.af, _segname, NULL, retdata, &buflen) != 0:
            raise IOError("error reading libaff segment")

        return bytes(retdata)

    """Retrieve a list of segments present"""
    def get_seg_names(self):
        headers = []
        segname = bytearray(afflib.AF_MAX_NAME_LEN)

        afflib.af_rewind_seg(self.af)
        while afflib.af_get_next_seg(self.af, segname, len(segname), NULL, NULL, NULL) == 0:
            if PY_MAJOR_VERSION < 3:
                headers.append(<char*> segname)
            else:
                headers.append((<char*> segname).decode('ascii'))

        return headers

    """Return possition within file"""
    def tell(self):
        return afflib.af_tell(self.af)

    """Close the file"""
    def close(self):
        afflib.af_close(self.af)

